#!/bin/bash
###########################################
# Software Requirement:                   #
#   bash x11-utils awk wmctrl bc xdotool  #
#   grep                                  #
# License:                                #
#   This script is licensed under GPL 3.0 #
# Version:                                #
#   0.47                                  #
# Author:                                 #
#   Alan CHENG <coolcd05a@gmail.com>      #
###########################################
## Custimization
# Path to essential commands
WMCTRL=`which wmctrl`
XDOTOOL=`which xdotool`
WST=(firefox-bin acroread x-www-browser google-chrome)
# Border width of windows decoration
BD_W=-1
# Titlebar height of windows decoration
TB_H=-1
# DISTANCE_HORI is the distance to move horizontally
DISTANCE_HORI=6
# DISTANCE_VERT is the distance to move vertically
DISTANCE_VERT=10
# DISTANCE_SLASH is the distance to move when using MOVE_UR, MOVE_UL, MOVE_DR, MOVE_DL
DISTANCE_SLASH=6
# DELAY is the time delay between each pixel movement
# PAUSE is the pause between MOVE_L, MOVE_D, MOVE_R, MOVE_U...
DELAY_HORI=0.05
DELAY_VERT=0.05
DELAY_SLASH=0.05
PAUSE=1
# Screen Width and Height
RESOLUTION=`wmctrl -d|awk '/\*/{ print $9}'`
SCR_W=`echo ${RESOLUTION} |awk -F x '{ print $1 }'`
SCR_H=`echo ${RESOLUTION} |awk -F x '{ print $2 }'`
#echo SCR_WxSCR_H="${SCR_W}"x"${SCR_H}"
# DEP_X and DEP_Y are Allowed Pixels outside the screen
# Pixels outside top and left edges don't work,
# this may be a bug/feature of wmctrl.
DEP_X=24
DEP_Y=24

## Functions
#  POS_WINDOWS() determins the position of windows
POS_WINDOWS () {
		ACT=`${XDOTOOL} getactivewindow`
		h=`echo "obase=16;${ACT}"|bc`
		WID_INIT=0x00000000
		WID_PREFIX=${WID_INIT:0:10-${#h}}
		w="${WID_PREFIX}""${h,,*}"
			#echo ACT="${ACT}"
			#echo w="$w"
			#L=(`${WMCTRL} -Gl`);
			#echo '$L'=$L;
			ORI_X=`xwininfo -id $w | awk '$0 ~ /Absolute upper-left X/{ print $4; }'`
			ORI_Y=`xwininfo -id $w | awk '$0 ~ /Absolute upper-left Y/{ print $4; }'`
			let ORI_X="${ORI_X}"-"${BD_W}"
			let ORI_Y="${ORI_Y}"-"${TB_H}"
			#echo '$CUR_Y'=${CUR_Y};
			#echo '$CUR_X'=${CUR_X};
}

#  ACT_CHK checks the process of the active window
ACT_CHK () {
		ACT_PID=`wmctrl -pl|awk '$1 ~ /'"$w"'/{ print $3 ; }'`
		ACT_PROCESS=`ps --no-header -o comm "${ACT_PID}"`
		echo "${WST[@]}"|grep "${ACT_PROCESS}" > /dev/null 2>&1
		a="$?"
		if [ "$a"q == 0q ]; then
			#echo You are currently running "${ACT_PROCESS}". The window will be swung.
			return 0
		else
			return 1
		fi
}

SIZE_WINDOWS () {
		CUR_W=`xwininfo -id $w | awk '$0 ~ /Width/{ print $2; }'`
		CUR_H=`xwininfo -id $w | awk '$0 ~ /Height/{ print $2; }'`
}

CHECK_INIT () {
		if [[ "${BD_W}" -ge 0 ]] &&  [[ "${TB_H}" -ge 0 ]] ;then
			CHECK=checked;
		else
			CHECK=unchecked;
			echo ""
			echo "You haven't initialized this script yet."
			echo "Please check titlebar height and border width settings first."
			echo ""
			echo "Step1: $./swing-windows.sh init"
			echo "Step2: Edit swing-windows.sh and input the values of BD_W and TB_H at the top of the script."
			echo ""
			echo "Now, please press ctrl+c to stop this script."
		fi
}

# Determine Titlebar height and Border width
DELTA () {
	ORI_X=`xwininfo -id $w | awk '$0 ~ /Absolute upper-left X/{ print $4; }'`
	ORI_Y=`xwininfo -id $w | awk '$0 ~ /Absolute upper-left Y/{ print $4; }'`
	${WMCTRL} -e 0,"${ORI_X}","${ORI_Y}",-1,-1 -i -r $w
	MOVED_X=`xwininfo -id $w | awk '$0 ~ /Absolute upper-left X/{ print $4; }'`
	MOVED_Y=`xwininfo -id $w | awk '$0 ~ /Absolute upper-left Y/{ print $4; }'`
	let BD_W="${MOVED_X}"-"${ORI_X}"
	let TB_H="${MOVED_Y}"-"${ORI_Y}"
	echo BD_W="${BD_W}"
	echo TB_H="${TB_H}"
}

# MOVE_R, MOVE_D, MOVE_L, MOVE_U move the window you selected
MOVE_R () {
	if [ $?q == 0q ];then
	for (( x=0 ; x<= ${DISTANCE_HORI}; x++ )) ;
		do let CUR_X="${ORI_X}"+"$x";
		CUR_Y="${ORI_Y}"
		if (( "${CUR_X}"+"${CUR_W}" >= "${SCR_W}"+"${DEP_X}" )) ;then
			return;
		else
			#echo '${CUR_X}'=${CUR_X}
			${WMCTRL} -e 0,${CUR_X},${CUR_Y},-1,-1 -i -r $w ;
			sleep ${DELAY_HORI};
		fi
		done;
	else
			return;
	fi
}

MOVE_D () {
	if [ $?q == 0q ];then
	for (( y=0 ; y<= ${DISTANCE_VERT}; y++ )) ;
		do let CUR_Y="${ORI_Y}"+"$y";
		CUR_X="${ORI_X}"
		if (( "${CUR_Y}"+"${CUR_H}" >= "${SCR_H}"+"${DEP_Y}" )) ;then
			return;
		else
			${WMCTRL} -e 0,${CUR_X},${CUR_Y},-1,-1 -i -r $w ;
			sleep ${DELAY_VERT};
		fi
		done;
	else
			return;
	fi
}

MOVE_L () {
	if [ $?q == 0q ];then
	for (( x=0 ; x <= ${DISTANCE_HORI}; x++ )) ;
		do let CUR_X="${ORI_X}"-"$x";
		CUR_Y="${ORI_Y}";
		if (( "${CUR_X}" <= 0-"${DEP_X}" )) ;then
			return;
		else
			${WMCTRL} -e 0,${CUR_X},${CUR_Y},-1,-1 -i -r $w ;
			sleep ${DELAY_HORI};
		fi
		done;
	else
			return;
	fi
}

MOVE_U () {
	if [ $?q == 0q ];then
	for (( y=0 ; y<= ${DISTANCE_VERT}; y++ )) ;
		do let CUR_Y="${ORI_Y}"-"$y";
		CUR_X="${ORI_X}"
		if (( "${CUR_Y}" <= 0-"${DEP_Y}" )) ;then
			return;
		else
			${WMCTRL} -e 0,${CUR_X},${CUR_Y},-1,-1 -i -r $w ;
			sleep ${DELAY_VERT};
		fi
		done;
	else
			return;
	fi
}

MOVE_UR () {
	if [ $?q == 0q ];then
	for (( y=0 ; y<= `echo "${DISTANCE_SLASH}/sqrt(2)"|bc` ; y++ )) ;
		do let CUR_Y="${ORI_Y}"-"$y";
		let CUR_X="${ORI_X}"+"$y";
		if (( "${CUR_Y}" <= 0-"${DEP_Y}" )) || (( "${CUR_X}" + "${CUR_W}" >= "${SCR_W}"+"${DEP_X}" )); then
			return;
		else
			${WMCTRL} -e 0,${CUR_X},${CUR_Y},-1,-1 -i -r $w ;
			sleep ${DELAY_SLASH};
		fi
		done;
	else
			return;
	fi
}

MOVE_UL () {
	if [ $?q == 0q ];then
	for (( y=0 ; y<= `echo "${DISTANCE_SLASH}/sqrt(2)"|bc` ; y++ )) ;
		do let CUR_Y="${ORI_Y}"-"$y";
		let CUR_X="${ORI_X}"-"$y";
		if (( "${CUR_Y}" <= 0-"${DEP_Y}" )) || (( "${CUR_X}" <= 0-"${DEP_X}" )); then
			return;
		else
			${WMCTRL} -e 0,${CUR_X},${CUR_Y},-1,-1 -i -r $w ;
			sleep ${DELAY_SLASH};
		fi
		done;
	else
			return;
	fi
}

MOVE_DR () {
	if [ $?q == 0q ];then
	for (( y=0 ; y<= `echo "${DISTANCE_SLASH}/sqrt(2)"|bc` ; y++ )) ;
		do let CUR_Y="${ORI_Y}"+"$y";
		let CUR_X="${ORI_X}"+"$y";
		if (( "${CUR_Y}" + "${CUR_H}" >= "${SCR_H}"+"${DEP_Y}" )) || (( "${CUR_X}" + "${CUR_W}" >= "${SCR_W}" + "${DEP_X}" )); then
			return;
		else
			${WMCTRL} -e 0,${CUR_X},${CUR_Y},-1,-1 -i -r $w ;
			sleep ${DELAY_SLASH};
		fi
		done;
	else
			return;
	fi
}

MOVE_DL () {
	if [ $?q == 0q ];then
	for (( y=0 ; y<= `echo "${DISTANCE_SLASH}/sqrt(2)"|bc` ; y++ )) ;
		do let CUR_Y="${ORI_Y}"+"$y";
		let CUR_X="${ORI_X}"-"$y";
		if (( "$CUR_Y" + "$CUR_H" >= "${SCR_H}" + "${DEP_Y}" )) || (( "$CUR_X" <= 0 - "${DEP_X}")); then
			return;
		else
			${WMCTRL} -e 0,${CUR_X},${CUR_Y},-1,-1 -i -r $w ;
			sleep ${DELAY_SLASH};
		fi
		done;
	else
			return;
	fi
}

case $1 in 
  init)
	echo "#######################################################"
	echo " WindowID    Window Title"
	echo ""
	${WMCTRL} -l|awk '{ print $1"  "$4" "$5" "$6" "$7  }'
	echo "#######################################################"
	echo "Please select the window for calculating the values of "
	echo "border width and titlebar height:"
	select w in `${WMCTRL} -l|awk '/^0x/{ print $1 }'`
	do 	W="$w" ;
		echo ""
		echo "You have selected $w to check" ;
		DELTA
		echo ""
		echo "Now, please press ctrl+c to exit and edit the"
		echo "values of the titlebar height (TB_H) and the"
		echo "border width (BD_W) at the top of the script."
		echo "Then, rerun this script."
	done
  ;;
  hori)
	CHECK_INIT;
	while [[ "$CHECK" == checked ]]
	do
	POS_WINDOWS;
	SIZE_WINDOWS
	ACT_CHK;
	MOVE_L;
	sleep ${PAUSE}
	POS_WINDOWS;
	ACT_CHK;
	MOVE_R;
	sleep ${PAUSE}
	done
  ;;
  vert)
	CHECK_INIT;
	while [[ "$CHECK" == checked ]]
	do
	POS_WINDOWS;
	SIZE_WINDOWS
	ACT_CHK;
	MOVE_D;
	sleep ${PAUSE}
	POS_WINDOWS;
	ACT_CHK;
	MOVE_U;
	sleep ${PAUSE}
	done
  ;;
  star)
	CHECK_INIT;
	while [[ "$CHECK" == checked ]]
	do
	POS_WINDOWS;
	SIZE_WINDOWS
	ACT_CHK;
	MOVE_U;
	sleep ${PAUSE}
	POS_WINDOWS;
	ACT_CHK;
	MOVE_D;
	sleep ${PAUSE}
	POS_WINDOWS;
	ACT_CHK;
	MOVE_L;
	sleep ${PAUSE}
	POS_WINDOWS;
	ACT_CHK;
	MOVE_R;
	sleep ${PAUSE}
	POS_WINDOWS;
	ACT_CHK;
	MOVE_R;
	sleep ${PAUSE}
	POS_WINDOWS;
	ACT_CHK;
	MOVE_L;
	sleep ${PAUSE}
	POS_WINDOWS;
	ACT_CHK;
	MOVE_DL;
	sleep ${PAUSE}
	POS_WINDOWS;
	ACT_CHK;
	MOVE_UR;
	sleep ${PAUSE}
	POS_WINDOWS;
	ACT_CHK;
	MOVE_DR;
	sleep ${PAUSE}
	POS_WINDOWS;
	ACT_CHK;
	MOVE_UL;
	sleep ${PAUSE}
	done
  ;;

  triangle-cw)
	CHECK_INIT;
	while [[ "$CHECK" == checked ]]
	do
	POS_WINDOWS;
	SIZE_WINDOWS;
	ACT_CHK;
	MOVE_DR;
	sleep ${PAUSE}
	POS_WINDOWS;
	ACT_CHK;
	MOVE_L;
	sleep ${PAUSE}
	POS_WINDOWS;
	ACT_CHK;
	MOVE_UR;
	sleep ${PAUSE}
	done;
;;
  triangle-ccw)
	CHECK_INIT;
	while [[ "$CHECK" == checked ]]
	do
	POS_WINDOWS;
	SIZE_WINDOWS;
	ACT_CHK;
	MOVE_DL;
	sleep ${PAUSE}
	POS_WINDOWS;
	ACT_CHK;
	MOVE_R;
	sleep ${PAUSE}
	POS_WINDOWS;
	ACT_CHK;
	MOVE_UL;
	sleep ${PAUSE}
	done;
;;
  square-cw)
	CHECK_INIT;
	while [[ "$CHECK" == checked ]]
	do
	POS_WINDOWS;
	SIZE_WINDOWS;
	ACT_CHK;
	MOVE_R;
	sleep ${PAUSE}
	POS_WINDOWS;
	ACT_CHK;
	MOVE_D;
	sleep ${PAUSE}
	POS_WINDOWS;
	ACT_CHK;
	MOVE_L;
	sleep ${PAUSE}
	POS_WINDOWS;
	ACT_CHK;
	MOVE_U;
	sleep ${PAUSE}
	done;
  ;;
  square-ccw)
	CHECK_INIT;
	while [[ "$CHECK" == checked ]]
	do
	POS_WINDOWS;
	SIZE_WINDOWS;
	ACT_CHK;
	MOVE_L;
	sleep ${PAUSE}
	POS_WINDOWS;
	ACT_CHK;
	MOVE_D;
	sleep ${PAUSE}
	POS_WINDOWS;
	ACT_CHK;
	MOVE_R;
	sleep ${PAUSE}
	POS_WINDOWS;
	ACT_CHK;
	MOVE_U;
	sleep ${PAUSE}
	done;
  ;;
  diamond-cw)
	CHECK_INIT;
	while [[ "$CHECK" == checked ]]
	do
	POS_WINDOWS;
	SIZE_WINDOWS;
	ACT_CHK;
	MOVE_DR;
	sleep ${PAUSE}
	POS_WINDOWS;
	ACT_CHK;
	MOVE_DL;
	sleep ${PAUSE}
	POS_WINDOWS;
	ACT_CHK;
	MOVE_UL;
	sleep ${PAUSE}
	POS_WINDOWS;
	ACT_CHK;
	MOVE_UR;
	sleep ${PAUSE}
	done;
  ;;
  diamond-ccw)
	CHECK_INIT;
	while [[ "$CHECK" == checked ]]
	do
	POS_WINDOWS;
	SIZE_WINDOWS;
	ACT_CHK;
	MOVE_DL;
	sleep ${PAUSE}
	POS_WINDOWS;
	ACT_CHK;
	MOVE_DR;
	sleep ${PAUSE}
	POS_WINDOWS;
	ACT_CHK;
	MOVE_UR;
	sleep ${PAUSE}
	POS_WINDOWS;
	ACT_CHK;
	MOVE_UL;
	sleep ${PAUSE}
	done;
  ;;
  octagon-cw)
	CHECK_INIT;
	while [[ "$CHECK" == checked ]]
	do
	POS_WINDOWS;
	SIZE_WINDOWS
	ACT_CHK;
	MOVE_R;
	sleep ${PAUSE}
	POS_WINDOWS
	ACT_CHK;
	MOVE_DR;
	sleep ${PAUSE}
	POS_WINDOWS
	ACT_CHK;
	MOVE_D;
	sleep ${PAUSE}
	POS_WINDOWS
	ACT_CHK;
	MOVE_DL;
	sleep ${PAUSE}
	POS_WINDOWS
	ACT_CHK;
	MOVE_L;
	sleep ${PAUSE}
	POS_WINDOWS
	ACT_CHK;
        MOVE_UL;
	sleep ${PAUSE}
	POS_WINDOWS
	ACT_CHK;
	MOVE_U;
	sleep ${PAUSE}
	POS_WINDOWS
	ACT_CHK;
	MOVE_UR;
	sleep ${PAUSE}
	done
  ;;
  octagon-ccw)
	CHECK_INIT;
	while [[ "$CHECK" == checked ]]
	do
	POS_WINDOWS;
	SIZE_WINDOWS
	ACT_CHK;
	MOVE_L;
	sleep ${PAUSE}
	POS_WINDOWS
	ACT_CHK;
	MOVE_DL;
	sleep ${PAUSE}
	POS_WINDOWS
	ACT_CHK;
	MOVE_D;
	sleep ${PAUSE}
	POS_WINDOWS
	ACT_CHK;
	MOVE_DR;
	sleep ${PAUSE}
	POS_WINDOWS
	ACT_CHK;
	MOVE_R;
	sleep ${PAUSE}
	POS_WINDOWS
	ACT_CHK;
        MOVE_UR;
	sleep ${PAUSE}
	POS_WINDOWS
	ACT_CHK;
	MOVE_U;
	sleep ${PAUSE}
	POS_WINDOWS
	ACT_CHK;
	MOVE_UL;
	sleep ${PAUSE}
	done
  ;;
  *)
	echo "Usage: /PATH_TO/swing-windows.sh {init|hori|vert|star|triangle-cw|triangle-ccw|square-cw|square-ccw|diamond-cw|diamond-ccw|octagon-cw|octagon-ccw}."
  ;;
esac
